"""This package defines types hints used by the streamunolib library."""

from typing import Any, Callable, Protocol, TypeVar, runtime_checkable

F = TypeVar('F', bound=Callable[..., Any])


@runtime_checkable
class ExposableCallable(Protocol[F]):
    """Interface of all components that can be exposed or hidden."""

    __call__: F
    __exposed__: bool



"""This package provides the decorators to annotate a transform component."""

from typing import Any, Callable, TypeVar, cast

from .types import ExposableCallable

F = TypeVar('F', bound=Callable[..., Any])


def exposed(f: F) -> ExposableCallable[F]:
    """Marks a function as a UI-visible transform component."""
    g = cast(ExposableCallable[F], f)
    g.__exposed__ = True
    return g


def hidden(f: F) -> ExposableCallable[F]:
    """Marks a function as a non UI-visible transform component."""
    g = cast(ExposableCallable[F], f)
    g.__exposed__ = False
    return g



"""This package contains tools to annotate transform outputs with a content type."""
from dataclasses import dataclass

COMMON_MIMETYPES = {
    'jpg': 'image/jpeg',
    'mp3': 'audio/mpeg',
    'mp4': 'video/mp4',
    'png': 'image/png',
    'svg': 'image/svg+xml',
    'wav': 'audio/wav',
}


@dataclass(frozen=True)
class ContentType:
    """The Content Type class."""

    mimetype: str


def content_type(mimetype: str) -> ContentType:
    """The ContentType factory.

    Parameters:
        mimetype: The MIME type used in the content type.

    Returns:
        The ContentType instance associated with the mime type.

    Examples:
        >>> content_type('wav')
        ContentType(mimetype='audio/wav')
        >>> content_type('text/csv')
        ContentType(mimetype='text/csv')
    """
    mimetype = mimetype.lower()
    try:
        mimetype = COMMON_MIMETYPES[mimetype]
    except KeyError:
        pass
    return ContentType(mimetype)



"""The streamUno library.

This library provides generic components to construct transforms and decorators
to annotate these components.
"""
from .content_types import content_type
from .decorators import exposed, hidden

__all__ = [
    'content_type',
    'exposed',
    'hidden',
]
